@@ -1,6 +1,6 @@
 /* Interface for accessing Gaisler AMBA Plug&Play Bus
  *
- * (C) Copyright 2011 Aeroflex Gaisler
+ * (C) Copyright 2011 Frontgrade Gaisler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -137,6 +137,10 @@ typedef struct ambapp_core_s ambapp_core;
 struct ambapp_core_s {
 	ambapp_core *next;
 	int index; /* Index of Core */
+#ifdef CONFIG_DEBUG
+	unsigned int vendor;
+	unsigned int device;
+#endif
 	int subindex; /* Index within set of cores with same vendor & device */
 
 	/* Device Information */
@@ -147,6 +151,7 @@ struct ambapp_core_s {
 	ambapp_core *next_in_drv;
 
 	void *fixup_priv; /* For carrying fixup data */
+	int fixup_versions;
 };
 
 struct ahb_bus {
@@ -209,28 +214,19 @@ extern int ambapp_for_each_dev(struct ambapp_bus *ainfo, int type,
 
 extern unsigned long ambapp_dev_freq(ambapp_dev *dev);
 
-/* AMBA Plug&Play Name of Vendors and devices */
-
-/* Return name of device */
-extern char *ambapp_device_id2str(int vendor, int id);
-
-/* Return name of vendor */
-extern char *ambapp_vendor_id2str(int vendor);
-
-/* Return description of device */
-extern const char *ambapp_device_id2desc(int vendor, int id);
-
-/* Return description of vendor */
-extern const char *ambapp_vendor_id2desc(int vendor);
-
 /* initialize a AHB device */
-extern void ambapp_dev_init_ahb(struct ahb_bus *bus, struct ambapp_pnp_ahb *ahb, ambapp_ahbdev *dev);
+extern void ambapp_dev_init_ahb(struct ahb_bus *bus,
+				struct ambapp_pnp_ahb *ahb,
+				ambapp_ahbdev *dev,
+				int got_version);
 
 #define DEV_NONE	0
 #define DEV_AHB_MST	1
 #define DEV_AHB_SLV	2
 #define DEV_APB_SLV	3
 
+#define DEV_VER_NONE    0xff
+
 #define AMBA_TYPE_APBIO 0x1
 #define AMBA_TYPE_MEM 0x2
 #define AMBA_TYPE_AHBIO 0x3
