@@ -1,6 +1,6 @@
 /* Exported functionality from PROM sections
  *
- * (C) Copyright 2011 Aeroflex Gaisler
+ * (C) Copyright 2011 Frontgrade Gaisler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -125,4 +125,9 @@ extern int _printf(const char *fmt, int a0, int a1, int a2, int a3, int a4,
 
 #endif /* CONFIG_DEBUG */
 
+struct systemid_banners {
+	unsigned long systemid;
+	char *banner;
+};
+
 #endif
