@@ -156,7 +156,10 @@ int canoc_ahb_fixup_virtual(ambapp_dev *dev, void *axb, void *bus, void **pdevp_
 			(*pdevp)++;
 		}
 		(*pdevp)->dev_type = DEV_AHB_SLV;
-		ambapp_dev_init_ahb((struct ahb_bus*)bus, (struct ambapp_pnp_ahb *)&ahb_buf, *pdevp);
+		ambapp_dev_init_ahb((struct ahb_bus*)bus,
+				    (struct ambapp_pnp_ahb *)&ahb_buf,
+				    *pdevp,
+				    1);
 		ahb_buf.id++;			/* Increase irq */
 		ahb_buf.mbar[0] += 0x100000;	/* Increase address */
 	}
@@ -668,6 +671,7 @@ struct amba_driver amba_drivers[] = {
  /*					   dev, core, prop, init */
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_IRQMP, irqmp_dev, NULL, NULL, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_GPTIMER, gptimer_apb_fixup, NULL, NULL, gptimer_init),
+ AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON5, NULL, NULL, NULL, leon_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON4, NULL, NULL, NULL, leon_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON3, NULL, NULL, NULL, leon_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON3FT, NULL, NULL, NULL, leon_init),
@@ -774,6 +778,35 @@ int ambapp_fixup_dev(ambapp_dev *dev)
 	return 0;
 }
 
+/*
+ * Fix up versions that are DEV_VER_NONE. Other than unset version
+ * fields all should already match at this point, no checks for that are done.
+ */
+static void ambapp_fixup_core_versions(ambapp_core *core)
+{
+	int version = DEV_VER_NONE;
+
+	/* Gather version */
+	if (core->ahbmst && core->ahbmst->ver != DEV_VER_NONE)
+		version = core->ahbmst->ver;
+	if (core->ahbslv && core->ahbslv->ver != DEV_VER_NONE)
+		version = core->ahbslv->ver;
+	if (core->apbslv && core->apbslv->ver != DEV_VER_NONE)
+		version = core->apbslv->ver;
+
+	/* If we have found none, set version to 0 */
+	if (version == DEV_VER_NONE)
+		version = 0;
+
+	/* Set version */
+	if (core->ahbmst)
+		core->ahbmst->ver = version;
+	if (core->ahbslv)
+		core->ahbslv->ver = version;
+	if (core->apbslv)
+		core->apbslv->ver = version;
+}
+
 /* Fixup AMBA PnP core information
  *
  * Return values:
@@ -786,6 +819,9 @@ int ambapp_fixup_core(ambapp_core *core)
 	ambapp_dev *dev;
 	int i;
 
+	if (core->fixup_versions)
+		ambapp_fixup_core_versions(core);
+
 	/* Find a device so that we get a VENDOR:DEVICE:VERSION */
 	if (core->ahbmst)
 		dev = (ambapp_dev *)core->ahbmst;
@@ -929,7 +965,16 @@ struct node *ahb2ahb_create_bus_node(struct ambapp_bus *ainfo, unsigned long ioa
 	return n;
 }
 
-void ahb2ahb_bus_create_iommu_fixup(
+/*
+ * Here the versions can be filled in as IOMMU master fields has no version
+ * information. They default to 0. Even though we can later on match a master
+ * with an unknown version to a slave with a known version, this is needed for
+ * when there is only a master and no slaves, e.g. for PCI DMA masters.
+ *
+ * Returns whether the version of the ahb struct is trustworthy after the call
+ * to this function.
+ */
+static int ahb2ahb_bus_create_iommu_fixup(
 	struct ambapp_bus *ainfo, struct ahb_bus *parent, ambapp_ahbdev *bridge,
 	struct ambapp_pnp_ahb *ahb, int index)
 {
@@ -939,11 +984,6 @@ void ahb2ahb_bus_create_iommu_fixup(
 	if (((systemid == GR740_REV0_SYSTEMID) ||
 	     (systemid == GR740_REV1_SYSTEMID)) &&
 	    (index <= 9)) {
-		/* GR740 reports always version 0 for all 10 masters.
-		 *  1 x PCI Master
-		 *  1 x PCI DMA Master
-		 *  4 x SpW masters
-		 */
 		int pciver;
 		int spwver = 1;
 
@@ -958,7 +998,15 @@ void ahb2ahb_bus_create_iommu_fixup(
 			*(unsigned int *)&ahb->id |= pciver << 5; /* PCI DMA master */
 		else if (index >= 4 && index <= 7)
 			*(unsigned int *)&ahb->id |= spwver << 5; /* SpaceWire masters */
+
+		/*
+		 * We can rely upon all versions, including the default zeroes
+		 * that we did not change.
+		 */
+		return 1;
 	}
+
+	return 0;
 }
 
 /* AHB-AHB IOMMU Creation */
@@ -1005,18 +1053,20 @@ void ahb2ahb_bus_create_iommu(struct ambapp_bus *ainfo, struct ahb_bus *parent,
 			_memcpy4_pa((void *)&mst, (void *)&regs->mstconf[j], 4);
 			if (mst) {
 				struct ambapp_pnp_ahb ahb_buf;
+				int gotver;
+
 				_memset((void*)&ahb_buf, 0, sizeof(ahb_buf));
 				*(int *)&(ahb_buf.id) = mst & 0xffffffe0;
 				*(int *)&(ahb_buf.mbar[0]) = 0; /* mst */
 
 				/* perform fixups */
-				ahb2ahb_bus_create_iommu_fixup(
-					ainfo, parent, bridge, &ahb_buf, j);
+				gotver = ahb2ahb_bus_create_iommu_fixup(
+						ainfo, parent, bridge, &ahb_buf, j);
 
 				/* Build a AHB Master Device */
 				pdev->dev_type = DEV_AHB_MST;
-				ambapp_dev_init_ahb(subbus, &ahb_buf, pdev);
-				
+				ambapp_dev_init_ahb(subbus, &ahb_buf, pdev, gotver);
+
 				if (ambapp_fixup_dev((ambapp_dev *)pdev) == 0) {
 					subbus->mst_cnt++;
 					pdev++;
@@ -1162,7 +1212,8 @@ static struct ambapp_bridge_driver *ambapp_bridge_detect(ambapp_ahbdev *dev)
 void ambapp_dev_init_ahb(
 	struct ahb_bus *bus,
 	struct ambapp_pnp_ahb *ahb,
-	ambapp_ahbdev *dev
+	ambapp_ahbdev *dev,
+	int got_version
 	)
 {
 	int bar;
@@ -1171,7 +1222,10 @@ void ambapp_dev_init_ahb(
 	/* Setup device struct */
 	dev->vendor = ambapp_pnp_vendor(ahb->id);
 	dev->device = ambapp_pnp_device(ahb->id);
-	dev->ver = ambapp_pnp_ver(ahb->id);
+	if (got_version)
+		dev->ver = ambapp_pnp_ver(ahb->id);
+	else
+		dev->ver = DEV_VER_NONE;
 	_memset(&dev->irq[0], 0, sizeof(dev->irq));
 	dev->extra = 0;
 	dev->irq[0] = ambapp_pnp_irq(ahb->id);
@@ -1297,7 +1351,7 @@ static void ambapp_scan_bus_ahb(
 
 		/* Build a AHB Master Device */
 		pdev->dev_type = DEV_AHB_MST;
-		ambapp_dev_init_ahb(bus, &ahb_buf, pdev);
+		ambapp_dev_init_ahb(bus, &ahb_buf, pdev, 1);
 
 		vfix = ambapp_fixup_virtual_dev((ambapp_dev *)pdev,
 						(void*)&ahb_buf, (void*)bus,
@@ -1337,7 +1391,7 @@ static void ambapp_scan_bus_ahb(
 
 		/* Build a AHB Slave Device */
 		pdev->dev_type = DEV_AHB_SLV;
-		ambapp_dev_init_ahb(bus, &ahb_buf, pdev);
+		ambapp_dev_init_ahb(bus, &ahb_buf, pdev, 1);
 
 		vfix = ambapp_fixup_virtual_dev((ambapp_dev *)pdev,
 						(void*)&ahb_buf, (void*)bus,
@@ -1500,6 +1554,11 @@ int ambapp_for_each_dev(struct ambapp_bus *ainfo, int type,
 	return 0; /* All Devices Processed */
 }
 
+static int matching_dev_versions(int a, int b)
+{
+	return a == b || a == DEV_VER_NONE || b == DEV_VER_NONE;
+}
+
 ambapp_core *ambapp_core_create(
 	struct ambapp_bus *ainfo,
 	ambapp_dev *dev,
@@ -1527,6 +1586,8 @@ ambapp_core *ambapp_core_create(
 	vendor = dev->vendor;
 	device = dev->device;
 	ver = dev->ver;
+	if (ver == DEV_VER_NONE)
+		core->fixup_versions = 1;
 
 #ifdef CONFIG_DEBUG
 	core->vendor = vendor;
@@ -1541,13 +1602,15 @@ ambapp_core *ambapp_core_create(
 			     j < ahbbus->slv_cnt; j++, ahb++) {
 				if ((vendor == ahb->vendor) &&
 				    (device == ahb->device) &&
-				    (ver == ahb->ver) &&
+				    matching_dev_versions(ver, ahb->ver) &&
 				    ((ahb->irq[0] & 0x80) == 0)) {
 					/* AHB Slave Device matching and not
 					 * marked taken.
 					 */
 					ahb->irq[0] |= 0x80;
 					core->ahbslv = ahb;
+					if (ahb->ver == DEV_VER_NONE)
+						core->fixup_versions = 1;
 					goto ahbscandone;
 				}
 			}
@@ -1563,13 +1626,15 @@ ahbscandone:
 			     j < apbbus->slv_cnt; j++, apb++) {
 				if ((vendor == apb->vendor) &&
 				    (device == apb->device) &&
-				    (ver == apb->ver) &&
+				    matching_dev_versions(ver, apb->ver) &&
 				    ((apb->irq[0] & 0x80) == 0)) {
 					/* AHB Slave Device matching and not
 					 * marked taken.
 					 */
 					apb->irq[0] |= 0x80;
 					core->apbslv = apb;
+					if (apb->ver == DEV_VER_NONE)
+						core->fixup_versions = 1;
 					goto apbscandone;
 				}
 			}
